<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class Mym00110InformativoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $dataSourceName = 'informativomymenu_filial_localpesq';
    protected $dataSourceNameToSave = 'informativomymenu';

    protected $fileServerFilesUrl;
    public function __construct($crudService, $parameterBag, $Mym00110InformativoService, $FCMService, $fileServerFilesUrl){
        parent::__construct($crudService, $parameterBag);
        $this->Mym00110InformativoService = $Mym00110InformativoService;
        $this->FCMService = $FCMService;
        $this->fileServerFilesUrl = $fileServerFilesUrl;
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = $this->dataSourceNameToSave ? : $this->dataSourceName;
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $rowsToSave = [];
            
            foreach ($rows as $row) {
                $fileColValue = null;
                $fileType = null;
                $url = $row['URL']??false;
                
                if (!$url) {
                    array_push($rowsToSave, $row);
                    continue;
                }
                
                // Processa strings diretamente
                if (is_string($url)) {
                    $fileColValue = str_replace($this->fileServerFilesUrl, "", $url);
                    $fileType = strpos($fileColValue, 'image') !== false ? 'image' : 'video';
                    
                  // Processa arrays de arquivos
                } elseif (is_array($url) && !empty($url[0])) {
                    $fileObject = $url[0];
                    if (is_array($fileObject) && isset($fileObject['type'], $fileObject['path'])) {
                        $fileType = substr($fileObject['type'], 0, 5); // Pega "image" ou "video"
                        $fileColValue = $fileObject['path'];
                    }
                }
                // Atribui o valor para a coluna correta, se tiver um tipo válido
                if ($fileType === 'image') {
                    $row['URLIMAGEM'] = $fileColValue;
                    $row['URLVIDEO'] = null;
                } elseif ($fileType === 'video') {
                    $row['URLVIDEO'] = $fileColValue;
                    $row['URLIMAGEM'] = null;
                }
                
                array_push($rowsToSave, $row);
            }
            $this->crudService->save($dataSourceName, $rowsToSave);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->setDataSourceName($this->dataSourceName);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
            
	        $responseDataSourceName = substr($request->getRoutePath(), 1);
	        $responseRows = [];
	        foreach ($response->getDataSets()[0]->getRows() as $row) {
                // Tratamento de URLIMAGEM
	            $imageCol = $row['URLIMAGEM'] ?? false;
	            if ($imageCol && is_string($imageCol)) {
	                $row['URLIMAGEM'] = $this->fileServerFilesUrl . $row['URLIMAGEM'];
	            }
	
	            // Tratamento de URLVIDEO
	            $videoCol = $row['URLVIDEO'] ?? false;
	            if ($videoCol && is_string($videoCol)) {
	                $row['URLVIDEO'] = $this->fileServerFilesUrl . $row['URLVIDEO'];
	            }
	
	            array_push($responseRows, $row);
	        }
	        
	        $responseDataSet = new DataSet($responseDataSourceName, $responseRows);
	        $response->setDataSets([$responseDataSet]);
	        
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function sendPushNotification(DTO\Request $request, DTO\Response $response) {
	    try{
	    	$row = $request->getRow();
	    	$CDFILIAL = $row['CDFILIAL'];
	    	$NRLOCALPESQ = isset($row['NRLOCALPESQ']) ? $row['NRLOCALPESQ'] : null; 
	    	$topicNames = $this->Mym00110InformativoService->getTopicNamesToPush($CDFILIAL, $NRLOCALPESQ);
	    	$datesExpired = $this->Mym00110InformativoService->getDatasExpiradasToPush($CDFILIAL, $NRLOCALPESQ);
	    	$title = $row['title'];
	    	$description = isset($row['description']) ? $row['description'] : "";
	    	$notification;
	    	if(!isset($row['NRINFORMATIVO'])){ // standard push notification without informativo bounded to it
				$notification = array(
				    "title" => $title, 
				    "body" => $description,
				    "data" => array(
				    "title" => $title, 
				    "body" => $description,
				    )
				);
	    	} else {
	    		$notification = array( // notification from a informativo
		    		"title" => $title, 
		    		"body" => $description,
		    		"data" => array(
		    			"NRINFORMATIVO" => 	$row['NRINFORMATIVO']
		    		)
		    	);
	    	}
	    	foreach ($topicNames as $topicName) {
		    	$this->FCMService->sendPushNotificationToTopic($topicName, $notification);
	    	}
    		$response->addDataSet(new DataSet("datasExpiradas", $datesExpired));
	    }catch(\Exception $e){
	    	$response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    
    
}